@@ -12,8 +12,8 @@ import xmltodict
 
 import abobjs
 
-class AB_Base:
 
+class AB_Base:
     # People need to set these three
     _endpoint = None
     _single = None
@@ -24,8 +24,8 @@ class AB_Base:
     # Base Stuff Override as needed
     _datefront = "%Y-%m-%dT%H:%M:%S"
     _mslength = 3
-    _date_keys = ["created_at", "updated_at", "deleted_at",	"effective_date",
-	              "baseline_date", "last_modification_date", "last_review_date"]
+    _date_keys = ["created_at", "updated_at", "deleted_at", "effective_date",
+                  "baseline_date", "last_modification_date", "last_review_date"]
 
     _create_null_keys = ["created_at", "deleted_at", "updated_at"]
     _read_args = dict()
@@ -84,7 +84,10 @@ class AB_Base:
 
         if self.kwargs.get("init_action", "read") in ("read", "readorcreate") and self.id is None:
             # I didn't get an ID
-            if self.kwargs.get("uid", None) is not None or self.kwargs.get("search_name", None) is not None:
+            if self.kwargs.get("uid", None) is not None or \
+                    self.kwargs.get("search_name", None) is not None or \
+                    self.kwargs.get( "search_complex", None) is not None:
+
                 # I searched for and found (or not) a result
                 self.id = self.search()
 
@@ -174,21 +177,19 @@ class AB_Base:
         search_endpoint = urllib.parse.urljoin(self.api_info.get("base_domain", None),
                                                self.endpoint)
 
-        if self.kwargs.get("all_data", None) is None:
-            # Grab control data
-            try:
-                data_request = requests.get(search_endpoint, headers=self.headers,
-                                            params=self.read_args)
-            except Exception as error:
-                self.logger.error("Error when requesting update about Control: {}".format(self.id))
-                self.logger.debug("Error Message: {}".format(error))
+        try:
+            data_request = requests.get(search_endpoint, headers=self.headers,
+                                        params=self.read_args)
+        except Exception as error:
+            self.logger.error("Error when requesting update about Control: {}".format(self.id))
+            self.logger.debug("Error Message: {}".format(error))
+        else:
+            if data_request.status_code != requests.codes.ok:
+                self.logger.error("Unable to Search {} for Data".format(self.name))
+                self.logger.debug(data_request.text)
+                raise ValueError()
             else:
-                if data_request.status_code != requests.codes.ok:
-                    self.logger.error("Unable to Search {} for Data".format(self.name))
-                    self.logger.debug(data_request.text)
-                    raise ValueError()
-                else:
-                    self.kwargs["all_data"] = data_request.json()
+                self.kwargs["all_data"] = data_request.json()
 
         found_item = None
         self.logger.debug("All JQ: {}".format(self.all_jq))
@@ -202,6 +203,37 @@ class AB_Base:
                 if potential.get(self.name_field, None) == self.kwargs["search_name"]:
                     found_item = potential
                     break
+            elif self.kwargs.get("search_complex", None) is not None:
+                all_passed = True
+                for k, v in self.kwargs["search_complex"].items():
+                    compare_value = potential.get("k", None)
+
+                    if isinstance(compare_value, list()):
+                        self.logger.error("List Values can't be compared at this time.")
+                        # TODO add in/only/not logic for lists
+                        all_passed = False
+                        break
+
+                    if isinstance(v, abobjs.AB_Base):
+                        if compare_value == v.id:
+                            # This one passed Continue to check the other items in search_complex
+                            pass
+                        else:
+                            all_passed = False
+                            break
+                    else:
+                        if compare_value == v:
+                            # This one Passed Continue to Check the other items in search_complex
+                            pass
+                        else:
+                            all_passed = False
+                            break
+
+                if all_passed is True:
+                    self.logger.debug("Found object via complex match {}".format(self.kwargs["search_complex"]))
+                    found_item = potential
+                    break
+
             else:
                 # Continue to the Next Item to search for it.
                 pass
@@ -231,7 +263,7 @@ class AB_Base:
         elif only_fields is None and self.allow_full_update is False:
             raise ValueError("Updates require specific field(s) for this type.")
         else:
-            put_data = {self.name: {k:v for k, v in all_put.items() if k in only_fields}}
+            put_data = {self.name: {k: v for k, v in all_put.items() if k in only_fields}}
 
         self.logger.debug(put_data)
 
@@ -307,13 +339,13 @@ class AB_Base:
                         for x in straight_data[key]:
                             replace_with.append(desired_obj["obj"](id=x, init_action="read",
                                                                    api_info=self.api_info,
-                                                                   depth=current_depth+1,
+                                                                   depth=current_depth + 1,
                                                                    my_user_id=self.my_user_id))
                     elif isinstance(straight_data[key], int):
                         # Replace Single Item
                         replace_with = desired_obj["obj"](id=straight_data[key], init_action="read",
                                                           api_info=self.api_info,
-                                                          depth=current_depth+1,
+                                                          depth=current_depth + 1,
                                                           my_user_id=self.my_user_id)
                     else:
                         replace_with = straight_data[key]
@@ -322,7 +354,6 @@ class AB_Base:
         else:
             self.logger.debug("Because of Depth {}, not Enumerating type : {}".format(current_depth, self.name))
 
-
         for key in self.date_keys:
             if key in objectified.keys():
                 if objectified[key] is not None:
@@ -342,7 +373,6 @@ class AB_Base:
 
         self.logger.debug(control_endpoint)
 
-
         try:
             data_request = requests.get(control_endpoint, headers=self.headers,
                                         params=self.read_args)
@@ -534,29 +564,29 @@ class AB_Base:
 
     def post_upload_post(self, file_name, form_data, response_data):
 
-        file_post_data = {"file":{
-                          "_permissions": {},
-                          "updated_at": None,
-                          "deleted_at": None,
-                          "local_folder_path": "",
-                          "comments": None,
-                          "embed_url": None,
-                          "thumb_url": None,
-                          "image_url": None,
-                          "meta": {},
-                          "creator_user_id": None,
-                          "created_at": self.datewrite(datetime.datetime.now()),
-                          "fileable_id": self.id,
-                          "fileable_type": self.fileable_type,
-                          "name": file_name,
-                          "size": "1",
-                          "type": "",
-                          "key": form_data["key"],
-                          "url": response_data["Location"],
-                          "storage_type": "s3",
-                          "user_agent": self.kwargs.get("user_agent", abobjs.USER_AGENT),
-                          "upload_user_id": "1"
-                         }}
+        file_post_data = {"file": {
+            "_permissions": {},
+            "updated_at": None,
+            "deleted_at": None,
+            "local_folder_path": "",
+            "comments": None,
+            "embed_url": None,
+            "thumb_url": None,
+            "image_url": None,
+            "meta": {},
+            "creator_user_id": None,
+            "created_at": self.datewrite(datetime.datetime.now()),
+            "fileable_id": self.id,
+            "fileable_type": self.fileable_type,
+            "name": file_name,
+            "size": "1",
+            "type": "",
+            "key": form_data["key"],
+            "url": response_data["Location"],
+            "storage_type": "s3",
+            "user_agent": self.kwargs.get("user_agent", abobjs.USER_AGENT),
+            "upload_user_id": "1"
+        }}
 
         fn_endpoint = urllib.parse.urljoin(self.api_info.get("base_domain", None),
                                            self.file_notify_endpoint)
